@@ -109,9 +109,9 @@ def write_keep_ids(pheno_path: str, fam_path: Optional[str], out_path: str) -> i
     non-NaN phenotype. If fam_path is provided, intersect with FAM; otherwise
     just emit from PHENO.
     """
-    ph = pd.read_csv(pheno_path, sep=r"\s+", header=None, usecols=[0,1,3],
-                     names=["FID","IID","value"], engine="python")
-    ph = ph[pd.to_numeric(ph["value"], errors="coerce").notna()][["FID","IID"]]
+    ph = pd.read_csv(pheno_path, sep=r"\s+", header=None, usecols=[0,1,2],
+                    names=["FID","IID","zscore"], engine="python")
+    ph = ph[pd.to_numeric(ph["zscore"], errors="coerce").notna()][["FID","IID"]]
 
     if fam_path:
         fam = pd.read_csv(fam_path, sep=r"\s+", header=None, usecols=[0,1],
@@ -187,37 +187,48 @@ def generate_pheno_plink_fast(
         map_df.to_csv(map_out, sep="\t", index=False, header=False)
 
         # Build per-strain queues (zscore,value) in this group's order
-        queues: Dict[str, List[Tuple[Optional[float], Optional[float]]]] = defaultdict(list)
+        # Need to make IID unique per animal_id to avoid duplicate id errors in PLINK.
+        queues: Dict[str, List[Tuple[str, Optional[float], Optional[float]]]] = defaultdict(list)
         for row in df.itertuples(index=False):
-            queues[str(row.strain)].append((getattr(row, "zscore", np.nan), getattr(row, "value", np.nan)))
+            strain = str(row.strain)
+            aid = getattr(row, "animal_id", None)
+            if aid is None or (isinstance(aid, float) and math.isnan(aid)):
+                # fallback to a stable per-strain counter when animal_id is missing
+                aid = f"rep{len(queues[strain]) + 1}"
+            aid_str = str(aid).replace(" ", "_").replace("/", "-")
+            queues[strain].append((aid_str, getattr(row, "zscore", np.nan), getattr(row, "value", np.nan)))
 
         unique_strains = list(dict.fromkeys(df["strain"].tolist()))  # preserves order
 
         ped_out = os.path.join(outdir, f"{measnum}.{sex}.ped")
         pheno_out = os.path.join(outdir, f"{measnum}.{sex}.pheno")
 
-        with open(ped_out, "w") as f_ped, open(pheno_out, "w") as f_ph:
+        with open(ped_out, "w", encoding="utf-8") as f_ped, open(pheno_out, "w", encoding="utf-8") as f_ph:
             for strain in unique_strains:
                 off = ped_index.get(strain)
                 if off is None:
-                    continue  # should not happen after filtering
+                    continue
                 line = _read_ped_line_at_offset(ped_file, off)
                 parts = _parse_tab_ped_line_to_parts(line)
                 if len(parts) >= 2:
-                    parts[0] = _sanitize_strain(parts[0])
-                    parts[1] = _sanitize_strain(parts[1])
+                    parts[0] = _sanitize_strain(parts[0])  # FID (strain)
+                    parts[1] = _sanitize_strain(parts[1])  # IID (will be overridden per animal)
+
+                for aid, z, v in queues[strain]:
+                    # --- make IID unique per animal while keeping FID=strain ---
+                    iid_unique = f"{strain}__{aid}"
+                    parts_mod = list(parts)
+                    parts_mod[1] = iid_unique
 
-                # For each phenotyped row in this strain, replicate:
-                for z, v in queues[strain]:
                     # PED: set SEX & PHE=zscore
-                    space_line = _flatten_pairs_to_space_stream(parts, sex_flag=sex, phe_value=z)
+                    space_line = _flatten_pairs_to_space_stream(parts_mod, sex_flag=sex, phe_value=z)
                     f_ped.write(space_line + "\n")
 
-                    # PHENO: FID IID zscore value
-                    fid = iid = strain
+                    # PHENO: FID IID zscore value (IID must match PED)
                     z_out = "" if (z is None or (isinstance(z, float) and math.isnan(z))) else str(z)
-                    v_out = "" if (v is None or (isinstance(v, float) and math.isnan(v))) else str(v)
-                    f_ph.write(f"{fid} {iid} {z_out} {v_out}\n")
+                    # v_out = "" if (v is None or (isinstance(v, float) and math.isnan(v))) else str(v)
+                    # f_ph.write(f"{strain} {iid_unique} {z_out} {v_out}\n")
+                    f_ph.write(f"{strain} {iid_unique} {z_out}\n")
 
     return ph
 
@@ -237,7 +248,7 @@ def fast_prepare_pylmm_inputs(
     ped_pheno_field: str = "zscore",  # accepted for API compatibility; ignored (we always write zscore to PED col6)
 ) -> None:
     """
-    Drop-in replacement for prepare_pylmm_inputs that uses the streaming PED writer
+    Fast replacement for prepare_pylmm_inputs that uses the streaming PED writer
     but produces the *same set of downstream PLINK outputs* as your current pipeline:
       - {base_id}.{sex}.ped/.map/.pheno
       - {base_id}.{sex}.bed/.bim/.fam
